<?php
/**
 * @file
 *  Include file for the "Site Install Hooks API" library.
 *
 *  © 2015 RedBottle Design, LLC and Inveniem.
 * Licensed under the GNU GPL v2 or later.
 *
 * @author Guy Paddock (guy.paddock@redbottledesign.com)
 */

/**
 * Constant for the machine name of this library.
 *
 * @var string
 */
define('SIIT_LIBRARY_NAME', 'site_install_hooks');

/**
 * <p>Initialize pre- and post-install hooks on behalf of the calling
 * installation profile.</p>
 *
 * <p>If your profile already defines a <code>hook_install_tasks_alter()</code>
 * implementation, you cannot use this function to initialize pre- and
 * post-install hooks.</p>
 */
function site_install_hooks_initialize($profile_name) {
  watchdog(
    SIIT_LIBRARY_NAME,
    'Initializing Site Install Hooks API.',
    array(),
    WATCHDOG_DEBUG);

  $code =
    <<<EOT
if (!function_exists('{$profile_name}_install_tasks_alter')) {
  function {$profile_name}_install_tasks_alter(&\$tasks, \$install_state) {
    site_install_hooks_install_tasks_alter(\$tasks, \$install_state);
  }
}
EOT;

  eval($code);
}

/**
 * <p>Pseudo-implementation of hook_install_tasks_alter().
 *
 * <p>This isn't a hook implementation, because Drupal will only invoke the hook
 * on the installation profile. Therefore, this method is invoked either by the
 * eval'd code defined by <code>site_install_hooks_initialize()</code> or
 * directly by a hook_install_tasks_alter() implementation in the installation
 * profile.</p>
 */
function site_install_hooks_install_tasks_alter(&$tasks, $install_state) {
  $internal_task = array(
    'display' => FALSE,
    'type'    => 'normal',
  );

  // Insert an internal task right before install_profile_modules.
  _site_install_hooks_array_insert_before(
    $tasks,
    'install_profile_modules',
    'site_install_hooks_invoke_pre_install',
    $internal_task);

  // The key in the array is the callback name, so the value is the same for
  // both the pre- and post-install tasks.
  $tasks['site_install_hooks_invoke_post_install'] = $internal_task;
}

/**
 * <p>Install callback that invokes <code>hook_site_pre_install()</code> on all
 * implementing modules.</p>
 *
 * <p>This hook is invoked immediately before
 * <code>install_profile_modules()</code> in Drupal core.</p>
 *
 * @param array $install_state
 *   A reference to the site installation state.
 */
function site_install_hooks_invoke_pre_install(&$install_state) {
  $profile_name = $install_state['parameters']['profile'];

  _site_install_hooks_invoke_install_hook_on_all_modules('site_pre_install', $profile_name);
}

/**
 * <p>Install callback that invokes hook_site_post_install() on all implementing
 * modules.</p>
 *
 * <p>This hook is invoked immediately after the last task in the install
 * profile (this is <code>install_configure_form()</code> for Drupal core's
 * profiles, but will vary from profile to profile).</p>
 *
 * @param array $install_state
 *   A reference to the site installation state.
 */
function site_install_hooks_invoke_post_install(&$install_state) {
  $profile_name = $install_state['parameters']['profile'];

  _site_install_hooks_invoke_install_hook_on_all_modules('site_post_install', $profile_name);
}

/**
 * Invokes the specified hook on all installed and enabled modules.
 *
 * @param string $hook_name
 *   The name of the hook to invoke.
 *
 * @param string $profile_name
 *   The machine name of the installation profile.
 */
function _site_install_hooks_invoke_install_hook_on_all_modules($hook_name, $profile_name) {
  /* Always include the installation profile in the list so its pre- and
   * post-install hooks can be invoked.
   */
  $modules = module_list() + array($profile_name => $profile_name);

  watchdog(
    SIIT_LIBRARY_NAME,
    'Invoking %hook on all modules',
    array('%hook' => $hook_name),
    WATCHDOG_DEBUG);

  foreach ($modules as $module_name) {
    /* Load .install file for the module, in case the pre-/post-install hook is
     * located there.
     */
    module_load_install($module_name);
    module_invoke($module_name, $hook_name);
  }
}

/**
 * <p>Inserts the provided key and value pair before the specified key in the
 * provided array.</p>
 *
 * <p>For example, consider this array:
 * <pre>
 *   array(
 *     'a' => 'apple',
 *     'c' => 'carrot',
 *   );
 * </pre>
 * </p>
 *
 * <p>Now, assume we call this method like so:
 * <code>_site_install_hooks_array_insert_before($array, 'c', 'b', 'banana')</code>
 * </p>
 *
 * <p>The array would now contain:
 * <pre>
 *   array(
 *     'a' => 'apple',
 *     'b' => 'banana',
 *     'c' => 'carrot',
 *   );
 * </pre>
 * </p>
 *
 * <p>If <code>$following_key</code> is not a key in <code>$array</code>, the
 * array is not modified.</p>
 *
 * @param array $array
 *   A reference to the associative array that is to be modified.
 *
 * @param mixed $following_key
 *   The key in $array before which the new key and value will be inserted.
 *
 * @param mixed $new_key
 *   The new key to insert before $following_key in $array.
 *
 * @param mixed $new_value
 *   The value to associate with $new_key in $array.
 */
function _site_install_hooks_array_insert_before(array &$array, $following_key, $new_key, $new_value) {
  $new_array = array();

  foreach ($array as $current_key => $current_value) {
    // Insert the new element before putting the target element in.
    if ($current_key === $following_key) {
      $new_array[$new_key] = $new_value;
    }

    $new_array[$current_key] = $current_value;
  }

  // Replace the contents of the passed-in array with the new array
  $array = $new_array;
}